"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertKeyBlock = exports.convertAwaitBlock = exports.convertEachBlock = exports.convertIfBlock = void 0;
const es_1 = require("./es");
const element_1 = require("./element");
const common_1 = require("./common");
/** Convert for IfBlock */
function convertIfBlock(node, parent, ctx) {
    // {#if expr} {:else} {/if}
    // {:else if expr} {/if}
    const nodeStart = node.elseif
        ? ctx.code.lastIndexOf("{", node.start)
        : node.start;
    const ifBlock = Object.assign({ type: "SvelteIfBlock", elseif: Boolean(node.elseif), expression: null, children: [], else: null, parent }, ctx.getConvertLocation({ start: nodeStart, end: node.end }));
    const es = es_1.convertESNode(node.expression, ifBlock, ctx);
    es_1.analyzeExpressionScope(es, ctx);
    ifBlock.expression = es;
    ctx.templateScopeManager.nestBlockScope(ifBlock, "block");
    ifBlock.children.push(...element_1.convertChildren(node, ifBlock, ctx));
    ctx.templateScopeManager.closeScope();
    if (node.elseif) {
        const index = ctx.code.indexOf("if", nodeStart);
        ctx.addToken("MustacheKeyword", { start: index, end: index + 2 });
    }
    extractMustacheBlockTokens(ifBlock, ctx, { startOnly: node.elseif });
    if (!node.else) {
        return ifBlock;
    }
    const elseStart = ctx.code.lastIndexOf("{", node.else.start);
    const elseBlock = Object.assign({ type: "SvelteElseBlock", children: [], parent: ifBlock }, ctx.getConvertLocation({
        start: elseStart,
        end: node.else.end,
    }));
    ifBlock.else = elseBlock;
    let elseIfBlock = null;
    if (node.else.children.length === 1) {
        const c = node.else.children[0];
        if (c.type === "IfBlock" && c.elseif) {
            elseIfBlock = convertIfBlock(c, elseBlock, ctx);
            // adjust loc
            elseBlock.range[1] = elseIfBlock.range[1];
            elseBlock.loc.end = {
                line: elseIfBlock.loc.end.line,
                column: elseIfBlock.loc.end.column,
            };
        }
    }
    if (elseIfBlock) {
        elseBlock.children.push(elseIfBlock);
    }
    else {
        ctx.templateScopeManager.nestBlockScope(elseBlock, "block");
        elseBlock.children.push(...element_1.convertChildren(node.else, elseBlock, ctx));
        ctx.templateScopeManager.closeScope();
        extractMustacheBlockTokens(elseBlock, ctx, { startOnly: true });
    }
    return ifBlock;
}
exports.convertIfBlock = convertIfBlock;
/** Convert for EachBlock */
function convertEachBlock(node, parent, ctx) {
    // {#each expr as item, index (key)} {/each}
    const eachBlock = Object.assign({ type: "SvelteEachBlock", expression: null, context: null, index: null, key: null, children: [], else: null, parent }, ctx.getConvertLocation(node));
    ctx.templateScopeManager.nestBlockScope(eachBlock, "for");
    const expression = es_1.convertESNode(node.expression, eachBlock, ctx);
    es_1.analyzeExpressionScope(expression, ctx);
    eachBlock.expression = expression;
    const asStart = ctx.code.indexOf("as", eachBlock.expression.range[1]);
    ctx.addToken("Keyword", {
        start: asStart,
        end: asStart + 2,
    });
    const context = es_1.convertESNode(node.context, eachBlock, ctx);
    es_1.analyzePatternScope(context, ctx);
    eachBlock.context = context;
    if (node.index) {
        const start = ctx.code.indexOf(node.index, eachBlock.context.range[1]);
        const rangeData = {
            start,
            end: start + node.index.length,
        };
        const index = Object.assign({ type: "Identifier", name: node.index, 
            // @ts-expect-error -- ignore
            parent: eachBlock }, ctx.getConvertLocation(rangeData));
        es_1.analyzePatternScope(index, ctx);
        eachBlock.index = index;
        ctx.addToken("Identifier", rangeData);
    }
    if (node.key) {
        const key = es_1.convertESNode(node.key, eachBlock, ctx);
        es_1.analyzeExpressionScope(key, ctx);
        eachBlock.key = key;
    }
    eachBlock.children.push(...element_1.convertChildren(node, eachBlock, ctx));
    ctx.templateScopeManager.closeScope();
    extractMustacheBlockTokens(eachBlock, ctx);
    if (!node.else) {
        return eachBlock;
    }
    const elseStart = ctx.code.lastIndexOf("{", node.else.start);
    const elseBlock = Object.assign({ type: "SvelteElseBlock", children: [], parent: eachBlock }, ctx.getConvertLocation({
        start: elseStart,
        end: node.else.end,
    }));
    eachBlock.else = elseBlock;
    ctx.templateScopeManager.nestBlockScope(elseBlock, "block");
    elseBlock.children.push(...element_1.convertChildren(node.else, elseBlock, ctx));
    ctx.templateScopeManager.closeScope();
    extractMustacheBlockTokens(elseBlock, ctx, { startOnly: true });
    return eachBlock;
}
exports.convertEachBlock = convertEachBlock;
/** Convert for AwaitBlock */
function convertAwaitBlock(node, parent, ctx) {
    const awaitBlock = Object.assign({ type: "SvelteAwaitBlock", expression: null, pending: null, then: null, catch: null, parent }, ctx.getConvertLocation(node));
    const expression = es_1.convertESNode(node.expression, awaitBlock, ctx);
    es_1.analyzeExpressionScope(expression, ctx);
    awaitBlock.expression = expression;
    if (!node.pending.skip) {
        const pendingBlock = Object.assign({ type: "SvelteAwaitPendingBlock", children: [], parent: awaitBlock }, ctx.getConvertLocation(node.pending));
        ctx.templateScopeManager.nestBlockScope(pendingBlock, "block");
        pendingBlock.children.push(...element_1.convertChildren(node.pending, pendingBlock, ctx));
        awaitBlock.pending = pendingBlock;
        ctx.templateScopeManager.closeScope();
    }
    if (!node.then.skip) {
        const thenStart = awaitBlock.pending ? node.then.start : node.start;
        const thenBlock = Object.assign({ type: "SvelteAwaitThenBlock", value: null, children: [], parent: awaitBlock }, ctx.getConvertLocation({
            start: thenStart,
            end: node.then.end,
        }));
        ctx.templateScopeManager.nestBlockScope(thenBlock, "block");
        const value = es_1.convertESNode(node.value, thenBlock, ctx);
        es_1.analyzePatternScope(value, ctx);
        thenBlock.value = value;
        thenBlock.children.push(...element_1.convertChildren(node.then, thenBlock, ctx));
        if (awaitBlock.pending) {
            extractMustacheBlockTokens(thenBlock, ctx, { startOnly: true });
        }
        else {
            const thenIndex = ctx.code.indexOf("then", expression.range[1]);
            ctx.addToken("MustacheKeyword", {
                start: thenIndex,
                end: thenIndex + 4,
            });
        }
        awaitBlock.then = thenBlock;
        ctx.templateScopeManager.closeScope();
    }
    if (!node.catch.skip) {
        const catchStart = awaitBlock.pending || awaitBlock.then
            ? node.catch.start
            : node.start;
        const catchBlock = Object.assign({ type: "SvelteAwaitCatchBlock", error: null, children: [], parent: awaitBlock }, ctx.getConvertLocation({
            start: catchStart,
            end: node.catch.end,
        }));
        ctx.templateScopeManager.nestBlockScope(catchBlock, "catch");
        const error = es_1.convertESNode(node.error, catchBlock, ctx);
        es_1.analyzePatternScope(error, ctx);
        catchBlock.error = error;
        catchBlock.children.push(...element_1.convertChildren(node.catch, catchBlock, ctx));
        if (awaitBlock.pending || awaitBlock.then) {
            extractMustacheBlockTokens(catchBlock, ctx, { startOnly: true });
        }
        else {
            const catchIndex = ctx.code.indexOf("catch", expression.range[1]);
            ctx.addToken("MustacheKeyword", {
                start: catchIndex,
                end: catchIndex + 5,
            });
        }
        awaitBlock.catch = catchBlock;
        ctx.templateScopeManager.closeScope();
    }
    extractMustacheBlockTokens(awaitBlock, ctx);
    return awaitBlock;
}
exports.convertAwaitBlock = convertAwaitBlock;
/** Convert for KeyBlock */
function convertKeyBlock(node, parent, ctx) {
    const keyBlock = Object.assign({ type: "SvelteKeyBlock", expression: null, children: [], parent }, ctx.getConvertLocation(node));
    const expression = es_1.convertESNode(node.expression, keyBlock, ctx);
    es_1.analyzeExpressionScope(expression, ctx);
    keyBlock.expression = expression;
    ctx.templateScopeManager.nestBlockScope(keyBlock, "block");
    keyBlock.children.push(...element_1.convertChildren(node, keyBlock, ctx));
    ctx.templateScopeManager.closeScope();
    extractMustacheBlockTokens(keyBlock, ctx);
    return keyBlock;
}
exports.convertKeyBlock = convertKeyBlock;
/** Extract mustache block tokens */
function extractMustacheBlockTokens(node, ctx, option) {
    const startSectionNameStart = common_1.indexOf(ctx.code, (c) => Boolean(c.trim()), node.range[0] + 1);
    const startSectionNameEnd = common_1.indexOf(ctx.code, (c) => c === "}" || !c.trim(), startSectionNameStart + 1);
    ctx.addToken("MustacheKeyword", {
        start: startSectionNameStart,
        end: startSectionNameEnd,
    });
    if (option === null || option === void 0 ? void 0 : option.startOnly) {
        return;
    }
    const endSectionNameEnd = common_1.lastIndexOf(ctx.code, (c) => Boolean(c.trim()), node.range[1] - 2) + 1;
    const endSectionNameStart = common_1.lastIndexOf(ctx.code, (c) => c === "{" || c === "/" || !c.trim(), endSectionNameEnd - 1);
    ctx.addToken("MustacheKeyword", {
        start: endSectionNameStart,
        end: endSectionNameEnd,
    });
}
