"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractElementTokens = exports.convertChildren = void 0;
const block_1 = require("./block");
const common_1 = require("./common");
const mustache_1 = require("./mustache");
const text_1 = require("./text");
const es_1 = require("./es");
const attr_1 = require("./attr");
/* eslint-disable complexity -- X */
/** Convert for Fragment or Element or ... */
function* convertChildren(
/* eslint-enable complexity -- X */
fragment, parent, ctx) {
    for (const child of fragment.children) {
        if (child.type === "Comment") {
            const comment = Object.assign({ type: "Block", value: child.data }, ctx.getConvertLocation(child));
            comment.html = true;
            ctx.addComment(comment);
            continue;
        }
        if (child.type === "Text") {
            yield text_1.convertText(child, parent, ctx);
            continue;
        }
        if (child.type === "Element") {
            yield convertHtmlElement(child, parent, ctx);
            continue;
        }
        if (child.type === "InlineComponent") {
            if (child.name.includes(":")) {
                yield convertSpecialElement(child, parent, ctx);
            }
            else {
                yield convertComponentElement(child, parent, ctx);
            }
            continue;
        }
        if (child.type === "Slot") {
            yield convertSlotElement(child, parent, ctx);
            continue;
        }
        if (child.type === "MustacheTag") {
            yield mustache_1.convertMustacheTag(child, parent, ctx);
            continue;
        }
        if (child.type === "RawMustacheTag") {
            yield mustache_1.convertRawMustacheTag(child, parent, ctx);
            continue;
        }
        if (child.type === "IfBlock") {
            // {#if expr} {/if}
            yield block_1.convertIfBlock(child, parent, ctx);
            continue;
        }
        if (child.type === "EachBlock") {
            // {#each expr as item, index (key)} {/each}
            yield block_1.convertEachBlock(child, parent, ctx);
            continue;
        }
        if (child.type === "AwaitBlock") {
            // {#await promise} {:then number} {:catch error} {/await}
            yield block_1.convertAwaitBlock(child, parent, ctx);
            continue;
        }
        if (child.type === "KeyBlock") {
            // {#key expression}...{/key}
            yield block_1.convertKeyBlock(child, parent, ctx);
            continue;
        }
        if (child.type === "Window") {
            yield convertWindowElement(child, parent, ctx);
            continue;
        }
        if (child.type === "Body") {
            yield convertBodyElement(child, parent, ctx);
            continue;
        }
        if (child.type === "Head") {
            yield convertHeadElement(child, parent, ctx);
            continue;
        }
        if (child.type === "Options") {
            yield convertOptionsElement(child, parent, ctx);
            continue;
        }
        if (child.type === "SlotTemplate") {
            yield convertSlotTemplateElement(child, parent, ctx);
            continue;
        }
        if (child.type === "DebugTag") {
            yield mustache_1.convertDebugTag(child, parent, ctx);
            continue;
        }
        throw new Error(`Unknown type:${child.type}`);
    }
}
exports.convertChildren = convertChildren;
/** Convert for HtmlElement */
function convertHtmlElement(node, parent, ctx) {
    const element = Object.assign({ type: "SvelteElement", kind: "html", name: null, attributes: [], children: [], parent }, ctx.getConvertLocation(node));
    element.attributes.push(...attr_1.convertAttributes(node.attributes, element, ctx));
    element.children.push(...convertChildren(node, element, ctx));
    extractElementTokens(element, ctx, {
        buildNameNode: (openTokenRange) => {
            const name = Object.assign({ type: "SvelteName", name: node.name, parent: element }, ctx.getConvertLocation(openTokenRange));
            return name;
        },
    });
    return element;
}
/** Convert for Special element. e.g. <svelte:self> */
function convertSpecialElement(node, parent, ctx) {
    const element = Object.assign({ type: "SvelteElement", kind: "special", name: null, attributes: [], children: [], parent }, ctx.getConvertLocation(node));
    element.attributes.push(...attr_1.convertAttributes(node.attributes, element, ctx));
    element.children.push(...convertChildren(node, element, ctx));
    if (node.type === "InlineComponent" &&
        node.expression &&
        node.name === "svelte:component") {
        const eqIndex = ctx.code.lastIndexOf("=", es_1.getWithLoc(node.expression).start);
        const startIndex = ctx.code.lastIndexOf("this", eqIndex);
        const closeIndex = ctx.code.indexOf("}", es_1.getWithLoc(node.expression).end);
        const endIndex = common_1.indexOf(ctx.code, (c) => c === ">" || !c.trim(), closeIndex);
        const thisAttr = Object.assign({ type: "SvelteSpecialDirective", kind: "this", expression: null, parent: element }, ctx.getConvertLocation({ start: startIndex, end: endIndex }));
        ctx.addToken("HTMLIdentifier", {
            start: startIndex,
            end: eqIndex,
        });
        const es = es_1.convertESNode(node.expression, thisAttr, ctx);
        es_1.analyzeExpressionScope(es, ctx);
        thisAttr.expression = es;
        element.attributes.push(thisAttr);
    }
    extractElementTokens(element, ctx, {
        buildNameNode: (openTokenRange) => {
            const name = Object.assign({ type: "SvelteName", name: node.name, parent: element }, ctx.getConvertLocation(openTokenRange));
            return name;
        },
    });
    return element;
}
/** Convert for ComponentElement */
function convertComponentElement(node, parent, ctx) {
    const element = Object.assign({ type: "SvelteElement", kind: "component", name: null, attributes: [], children: [], parent }, ctx.getConvertLocation(node));
    element.attributes.push(...attr_1.convertAttributes(node.attributes, element, ctx));
    element.children.push(...convertChildren(node, element, ctx));
    extractElementTokens(element, ctx, {
        buildNameNode: (openTokenRange) => {
            const name = Object.assign({ type: "Identifier", name: node.name, 
                // @ts-expect-error -- ignore
                parent: element }, ctx.getConvertLocation(openTokenRange));
            return name;
        },
    });
    es_1.analyzeExpressionScope(element.name, ctx);
    return element;
}
/** Convert for Slot */
function convertSlotElement(node, parent, ctx) {
    // Slot translates to SvelteHtmlElement.
    return convertHtmlElement(node, parent, ctx);
}
/** Convert for window element. e.g. <svelte:window> */
function convertWindowElement(node, parent, ctx) {
    return convertSpecialElement(node, parent, ctx);
}
/** Convert for body element. e.g. <svelte:body> */
function convertBodyElement(node, parent, ctx) {
    return convertSpecialElement(node, parent, ctx);
}
/** Convert for head element. e.g. <svelte:head> */
function convertHeadElement(node, parent, ctx) {
    return convertSpecialElement(node, parent, ctx);
}
/** Convert for options element. e.g. <svelte:options> */
function convertOptionsElement(node, parent, ctx) {
    return convertSpecialElement(node, parent, ctx);
}
/** Convert for <svelte:fragment> element. */
function convertSlotTemplateElement(node, parent, ctx) {
    return convertSpecialElement(node, parent, ctx);
}
/** Extract element block tokens */
function extractElementTokens(element, ctx, options) {
    const startTagNameEnd = common_1.indexOf(ctx.code, (c) => c === "/" || c === ">" || !c.trim(), element.range[0] + 1);
    const openTokenRange = {
        start: element.range[0] + 1,
        end: startTagNameEnd,
    };
    const openToken = ctx.addToken("HTMLIdentifier", openTokenRange);
    element.name = options.buildNameNode(openTokenRange);
    if (ctx.code[element.range[1] - 1] !== ">") {
        // Have not end tag
        return;
    }
    if (ctx.code[element.range[1] - 2] === "/") {
        // self close
        return;
    }
    const attrEnd = element.attributes.length
        ? element.attributes[element.attributes.length - 1].range[1]
        : openToken.range[1];
    const endTagOpen = ctx.code.lastIndexOf("<", element.range[1] - 1);
    if (endTagOpen <= attrEnd) {
        // void element
        return;
    }
    const endTagNameStart = endTagOpen + 2;
    const endTagNameEnd = common_1.indexOf(ctx.code, (c) => c === ">" || !c.trim(), endTagNameStart);
    ctx.addToken("HTMLIdentifier", {
        start: endTagNameStart,
        end: endTagNameEnd,
    });
}
exports.extractElementTokens = extractElementTokens;
